@@ -21,8 +21,8 @@
 ## Criterion for the quality of a permutation of a matrix
 
 criterion.matrix <-
-function(x, order = NULL, method = NULL, ...)
-  .criterion_array_helper(x, order, method, "matrix") 
+function(x, order = NULL, method = NULL, force_loss = FALSE, ...)
+  .criterion_array_helper(x, order, method, "matrix", force_loss)
 
 ## Bond energy (BEA)
 criterion_ME <- function(x, order = NULL, ...) {
@@ -30,12 +30,12 @@ criterion_ME <- function(x, order = NULL, ...) {
 
     if(any(x < 0))
         stop("Bond energy (ME) is only defined for nonnegative matrices.")
-    
+
     n <- nrow(x)
     m <- ncol(x)
-    
+
     if(!is.null(order)) x <- permute(x, order)
-    
+
     mode(x) <- "single"
 
     energy <- .Fortran("energy",
@@ -44,7 +44,7 @@ criterion_ME <- function(x, order = NULL, ...) {
                        b = x,
                        ener = as.single(0.0),
 		       PACKAGE="seriation")
-    
+
     0.5 * as.numeric(energy$ener)
 }
 
@@ -60,7 +60,7 @@ criterion_ME <- function(x, order = NULL, ...) {
     if (!is.matrix(x))
         stop("Argument 'x' must be a matrix.")
     if (!is.double(x)) mode(x) <- "double"
-    
+
     if(is.null(order)) {
         rows <- as.integer(1:dim(x)[1])
         cols <- as.integer(1:dim(x)[2])
@@ -68,11 +68,11 @@ criterion_ME <- function(x, order = NULL, ...) {
         rows <- get_order(order, 1)
         cols <- get_order(order, 2)
     }
-    
+
     type <- as.integer(TYPE[type])
-    
+
     x <- .Call("stress", x, rows, cols, type)
-    
+
     ## does only half of the matrix!
     2 * x
 }
@@ -80,9 +80,45 @@ criterion_ME <- function(x, order = NULL, ...) {
 criterion_stress_moore <- function(x, order, ...) .stress(x, order, "moore")
 criterion_stress_neumann <- function(x, order, ...) .stress(x, order, "neumann")
 
+### A MEASURE OF EFFECTIVENESS FOR THE MOMENT ORDERING ALGORITHM
+### by Deutsch & Martin (1971)
+### Correlation coefficient R for matrices.
+criterion_R_matrix <- function(x, order, ...) {
+  if(!is.null(order)) x <- permute(x, order)
+
+  M <- nrow(x)
+  N <- ncol(x)
+
+  ## total sum
+  T <- sum(x)
+
+  ## X_i = i/M; Y_j = j/N
+  X_i <- (1:M)/M
+  Y_j <- (1:N)/N
+
+  ## X_bar = 1/T sum_i,j a_ij X_i
+  X_bar <- 1/T * sum(crossprod(x, X_i))
+  ## Y_bar = 1/T sum_i,j a_ij Y_j
+  Y_bar <- 1/T * sum(crossprod(t(x), Y_j))
+
+  ## S_X2 = 1/(T-1) sum_i,j a_ij (X_i - X_bar)^2
+  S_X2 <- 1/(T-1) * sum(crossprod(x, (X_i-X_bar)^2))
+  ## S_Y2 = 1/(T-1) sum_i,j a_ij (Y_j - Y_bar)^2
+  S_Y2 <- 1/(T-1) * sum(crossprod(t(x), (Y_j-Y_bar)^2))
+
+  ## S_XY = 1/(T-1) sum_i,j a_ij  (X_i - X_bar) (Y_j - Y_bar)
+  S_XY <- 1/(T-1) * sum(x * outer(X_i-X_bar, Y_j-Y_bar))
+
+  ## R = S_XY/(S_X S_Y)
+  S_XY/(sqrt(S_X2)*sqrt(S_Y2))
+}
+
 ## register built-ins
-set_criterion_method("matrix", "ME", criterion_ME, 
+set_criterion_method("matrix", "ME", criterion_ME,
     "Measure of effectiveness", TRUE)
+set_criterion_method("matrix", "Cor_R", criterion_R_matrix,
+    "Correlation Coefficient R", TRUE)
+
 set_criterion_method("matrix", "Moore_stress", criterion_stress_moore,
     "Stress (Moore neighborhood)", FALSE)
 set_criterion_method("matrix", "Neumann_stress", criterion_stress_neumann,
